@@ -1,3 +1,23 @@
+/*
+ *  Copyright (C) 2003, 2004 Red Hat, Inc.
+ *
+ *  Nautilus is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU General Public License as
+ *  published by the Free Software Foundation; either version 2 of the
+ *  License, or (at your option) any later version.
+ *
+ *  Nautilus is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *  Authors: Alexander Larsson <alexl@redhat.com>
+ *
+ */
 #include <config.h>
 #include <libbonobo.h>
 #include <bonobo/bonobo-main.h>
@@ -14,7 +34,7 @@
 
 /* Global daemon */
 static GnomeVFSDaemon *the_daemon = NULL;
-static GnomeVFSAsyncDaemon *async_daemon = NULL;
+static GnomeVFSAsyncDaemon *the_async_daemon = NULL;
 
 typedef struct {
 	GNOME_VFS_Client client;
@@ -186,7 +206,7 @@ volume_mounted (GnomeVFSVolumeMonitor *volume_monitor,
 
 	corba_volume = GNOME_VFS_Volume__alloc ();
 
-	_gnome_vfs_volume_to_corba (volume, corba_volume);
+	gnome_vfs_volume_to_corba (volume, corba_volume);
 	
 	CORBA_exception_init (&ev);
 	GNOME_VFS_Client_VolumeMounted (client,
@@ -244,7 +264,7 @@ drive_connected (GnomeVFSVolumeMonitor *volume_monitor,
 
 	corba_drive = GNOME_VFS_Drive__alloc ();
 
-	_gnome_vfs_drive_to_corba (drive, corba_drive);
+	gnome_vfs_drive_to_corba (drive, corba_drive);
 	
 	CORBA_exception_init (&ev);
 	GNOME_VFS_Client_DriveConnected (client,
@@ -340,7 +360,7 @@ get_volumes (PortableServer_Servant _servant,
 	list->_maximum = len;
 
 	for (i = 0, l = volumes; l != NULL; l = l->next, i++) {
-		_gnome_vfs_volume_to_corba (l->data, &list->_buffer[i]);
+		gnome_vfs_volume_to_corba (l->data, &list->_buffer[i]);
 		gnome_vfs_volume_unref (l->data);
 	}
 	g_list_free (volumes);
@@ -369,7 +389,7 @@ get_drives (PortableServer_Servant _servant,
 	list->_maximum = len;
 
 	for (i = 0, l = drives; l != NULL; l = l->next, i++) {
-		_gnome_vfs_drive_to_corba (l->data, &list->_buffer[i]);
+		gnome_vfs_drive_to_corba (l->data, &list->_buffer[i]);
 		gnome_vfs_drive_unref (l->data);
 	}
 	g_list_free (drives);
@@ -390,8 +410,8 @@ emit_pre_unmount_volume (PortableServer_Servant _servant,
 
 	volume = gnome_vfs_volume_monitor_get_volume_by_id (monitor, id);
 	if (volume != NULL) {
-		_gnome_vfs_volume_monitor_emit_pre_unmount (monitor,
-							    volume);
+		gnome_vfs_volume_monitor_emit_pre_unmount (monitor,
+							   volume);
 		gnome_vfs_volume_unref (volume);
 	}
 }
@@ -534,12 +554,12 @@ gnome_vfs_daemon_factory (BonoboGenericFactory *factory,
 		the_daemon = g_object_new (GNOME_TYPE_VFS_DAEMON, NULL);
 
 		poa = bonobo_poa_get_threaded (ORBIT_THREAD_HINT_PER_REQUEST);
-		async_daemon = g_object_new (GNOME_TYPE_VFS_ASYNC_DAEMON,
+		the_async_daemon = g_object_new (GNOME_TYPE_VFS_ASYNC_DAEMON,
 					     "poa", poa,
 					     NULL);
 		CORBA_Object_release ((CORBA_Object)poa, NULL);
 		bonobo_object_add_interface (BONOBO_OBJECT (the_daemon),
-					     BONOBO_OBJECT (async_daemon));
+					     BONOBO_OBJECT (the_async_daemon));
 	}
 	return BONOBO_OBJECT (the_daemon);
 }
